@@ -1,7 +1,9 @@
 package Perinci::Sub::To::CLIDocData;
 
-our $DATE = '2019-07-26'; # DATE
-our $VERSION = '0.291'; # VERSION
+our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
+our $DATE = '2020-04-27'; # DATE
+our $DIST = 'Perinci-Sub-To-CLIDocData'; # DIST
+our $VERSION = '0.292'; # VERSION
 
 use 5.010001;
 use strict;
@@ -184,49 +186,8 @@ sub gen_cli_doc_data_from_meta {
         example_categories => {},
     };
 
-    # generate usage line
-    {
-        my @args;
-        my %args_prop = %$args_prop; # copy because we want to iterate & delete
-        my $max_pos = -1;
-        for (values %args_prop) {
-            $max_pos = $_->{pos}
-                if defined($_->{pos}) && $_->{pos} > $max_pos;
-        }
-        my $pos = 0;
-        while ($pos <= $max_pos) {
-            my ($arg, $arg_spec);
-            for (keys %args_prop) {
-                $arg_spec = $args_prop{$_};
-                if (defined($arg_spec->{pos}) && $arg_spec->{pos}==$pos) {
-                    $arg = $_;
-                    last;
-                }
-            }
-            $pos++;
-            next unless defined($arg);
-            if ($arg_spec->{slurpy} // $arg_spec->{greedy}) {
-                # try to find the singular form
-                $arg = $arg_spec->{'x.name.singular'}
-                    if $arg_spec->{'x.name.is_plural'} &&
-                    defined $arg_spec->{'x.name.singular'};
-            }
-            if ($arg_spec->{req}) {
-                push @args, "<$arg>";
-            } else {
-                push @args, "[$arg]";
-            }
-            $args[-1] .= " ..." if ($arg_spec->{slurpy} // $arg_spec->{greedy});
-            delete $args_prop{$arg};
-        }
-        unshift @args, "[options]" if keys(%args_prop) || keys(%$common_opts); # XXX translatable?
-        $clidocdata->{usage_line} = "[[prog]]".
-            (@args ? " ".join(" ", @args) : "");
-    }
-
-    # generate list of options
     my %opts;
-    {
+  GEN_LIST_OF_OPTIONS: {
         my $ospecs = $ggls_res->[3]{'func.specmeta'};
         # separate groupable aliases because they will be merged with the
         # argument options
@@ -426,9 +387,69 @@ sub gen_cli_doc_data_from_meta {
             }
         }
 
-    }
+    } # GEN_LIST_OF_OPTIONS
     $clidocdata->{opts} = \%opts;
 
+  GEN_USAGE_LINE: {
+        my @args;
+        my %args_prop = %$args_prop; # copy because we want to iterate & delete
+        my $max_pos = -1;
+        for (values %args_prop) {
+            $max_pos = $_->{pos}
+                if defined($_->{pos}) && $_->{pos} > $max_pos;
+        }
+        my $pos = 0;
+        while ($pos <= $max_pos) {
+            my ($arg, $arg_spec);
+            for (keys %args_prop) {
+                $arg_spec = $args_prop{$_};
+                if (defined($arg_spec->{pos}) && $arg_spec->{pos}==$pos) {
+                    $arg = $_;
+                    last;
+                }
+            }
+            $pos++;
+            next unless defined($arg);
+            if ($arg_spec->{slurpy} // $arg_spec->{greedy}) {
+                # try to find the singular form
+                $arg = $arg_spec->{'x.name.singular'}
+                    if $arg_spec->{'x.name.is_plural'} &&
+                    defined $arg_spec->{'x.name.singular'};
+            }
+            if ($arg_spec->{req}) {
+                push @args, "<$arg>";
+            } else {
+                push @args, "[$arg]";
+            }
+            $args[-1] .= " ..." if ($arg_spec->{slurpy} // $arg_spec->{greedy});
+            delete $args_prop{$arg};
+        }
+
+        # XXX utilize information from args_rels
+
+        require Getopt::Long::Util;
+        my @opts;
+        for my $ospec (sort keys %{ $ggls_res->[3]{'func.specmeta'} }) {
+            my $ospecmeta = $ggls_res->[3]{'func.specmeta'}{$ospec};
+            my $argprop = defined $ospecmeta->{arg} ? $args_prop{ $ospecmeta->{arg} } : undef;
+            # only include args that have not been mentioned in positional
+            next if defined $ospecmeta->{arg} && !$argprop;
+            # only inlude common options that are not a specific action that are
+            # invoked on its own
+            next if defined $ospecmeta->{common_opt} && $common_opts->{ $ospecmeta->{common_opt} }{usage};
+            push @opts, "[".Getopt::Long::Util::humanize_getopt_long_opt_spec({
+                separator=>" | ",
+                value_label=>($argprop->{'x.cli.opt_value_label'} // $argprop->{caption}),
+            }, $ospec)."]";
+        }
+
+        $clidocdata->{compact_usage_line} = "[[prog]]".
+            (keys(%args_prop) || keys(%$common_opts) ? " [options]" : ""). # XXX translatable?
+            (@args ? " ".join(" ", @args) : "");
+        $clidocdata->{usage_line} = "[[prog]]".
+            (@args ? " ".join(" ", @opts, @args) : "");
+    } # GEN_USAGE_LINE
+
     # filter and format examples
     my @examples;
     {
@@ -496,7 +517,7 @@ Perinci::Sub::To::CLIDocData - From Rinci function metadata, generate structure
 
 =head1 VERSION
 
-This document describes version 0.291 of Perinci::Sub::To::CLIDocData (from Perl distribution Perinci-Sub-To-CLIDocData), released on 2019-07-26.
+This document describes version 0.292 of Perinci::Sub::To::CLIDocData (from Perl distribution Perinci-Sub-To-CLIDocData), released on 2020-04-27.
 
 =head1 SYNOPSIS
 
@@ -543,6 +564,7 @@ Sample result:
      200,
      "OK",
      {
+       compact_usage_line => "[[prog]] [options] <str1>",
        example_categories => { Examples => { order => 99 } },
        examples => [
          {
@@ -628,7 +650,7 @@ Sample result:
            tags        => 'fix',
          },
        },
-       usage_line => "[[prog]] [options] <str1>",
+       usage_line => "[[prog]] [--bool1] [-f] [--flag1] [--no-bool1] [--nobool1] [-z] <str1>",
      },
    ];
    $a->[2]{opts}{"--bool1"}{tags} = $a->[2]{opts}{"--bool1"}{arg_spec}{tags};
@@ -642,6 +664,10 @@ Sample result:
 For a more complete sample, see function metadata for C<demo_cli_opts> in
 L<Perinci::Examples::CLI>.
 
+=head1 RINCI
+
+Observed function argument attribute: C<x.cli.opt_value_label>, C<caption>, C<>.
+
 =head1 FUNCTIONS
 
 
@@ -651,7 +677,7 @@ Usage:
 
  gen_cli_doc_data_from_meta(%args) -> [status, msg, payload, meta]
 
-From Rinci function metadata, generate structure convenient for producing CLI documentation (help/usage/POD).
+From Rinci function metadata, generate structure convenient for producing CLI documentation (helpE<sol>usageE<sol>POD).
 
 This function calls L<Perinci::Sub::GetArgs::Argv>'s
 C<gen_getopt_long_spec_from_meta()> (or receive its result as an argument, if
@@ -696,6 +722,7 @@ Pass per_arg_json=1 to Perinci::Sub::GetArgs::Argv.
 
 Pass per_arg_json=1 to Perinci::Sub::GetArgs::Argv.
 
+
 =back
 
 Returns an enveloped result (an array).
@@ -737,7 +764,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2019, 2016, 2015, 2014 by perlancar@cpan.org.
+This software is copyright (c) 2020, 2019, 2016, 2015, 2014 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
